# -*- coding: utf-8 -*-
"""
Created on Tue Mar 2 11:26:05 2021

@author: Mateus A. R. Braga

Cost Algorithm (Revision 2)
"""
# --------
# Imports
# --------
import math
#import numpy as np
import matplotlib.pyplot as plt
import timeit
from datetime import datetime
import traceback

startLog = datetime.now()
start = timeit.default_timer() # Starting the timer to compute the runtime

try:
    from ReadOTSCfiles import (capacity_battery_dict, voltage_battery_dict, 
    mass_battery_dict, cost_battery_dict, numRotors_frame_dict, 
    wheelbase_frame_dict, mass_frame_dict, cost_frame_dict, Kv_motor_dict, 
    Imax_motor_dict, Vmax_motor_dict, mass_motor_dict, cost_motor_dict, 
    diameter_propeller_dict, pitch_propeller_dict, mass_propeller_dict, 
    cost_propeller_dict, Vmin_speedcontroller_dict, Vmax_speedcontroller_dict, 
    Imax_speedcontroller_dict, mass_speedcontroller_dict,cost_speedcontroller_dict)
    
    
    # --------
    # Inputs
    # --------
    mpld = 0 #[kg] mass of the payload
    # Environmental and Flight Conditions
    g = 9.81 # [m/s^2] acceleration due to gravity
    rho = 1.225 # [kg/m^3] air density
    t = 10/60 # [h] flight time
    
    
    # ---------------------------------------
    # Cycling Through Component Combinations
    # ---------------------------------------
    numB = len(mass_battery_dict) # Number of different batteries available
    numF = len(mass_frame_dict) # Number of different frames available 
    numM = len(mass_motor_dict) # Number of different motors available
    numP = len(mass_frame_dict) # Number of different propellers available 
    numSC = len(mass_speedcontroller_dict) # Num diff speed controllers available
    
    
    n = 4 # number of motors, speed controllers, and propellers
    #*** n can only be 4, 6, or 8 so that there is a compatible frame!!!
    while n <= 8:
        solutions_dict = {}
        print("\n\nn = %d"%(n))
        print("------------")
        print("------------")
        for i in range(numB):
            print("\nBattery b%s"%(i+1))
            print("------------")
        
            # Battery Parameters
            cap = 0.8*capacity_battery_dict['cap_b%s'%(i+1)] #[mAh] capacity for one battery (never going below 20% charge)
            Vb = voltage_battery_dict['V_b%s'%(i+1)] #[V] battery voltage rating
            mb = mass_battery_dict['m_b%s'%(i+1)]/1000 #[kg] mass of one battery
            costb = cost_battery_dict['cost_b%s'%(i+1)] #[USD] cost of one battery
            
            for j in range(numP):
                print("\nPropeller p%s"%(j+1))
                print("------------")  
            
                # Propeller Parameters
                pitch = pitch_propeller_dict['pitch_p%s'%(j+1)]*0.0254 # [m] propeller pitch
                d = diameter_propeller_dict['d_p%s'%(j+1)]*0.0254 # [m] propeller diameter
                mp = mass_propeller_dict['m_p%s'%(j+1)]/1000 #[kg] mass of one propeller
                costp = cost_propeller_dict['cost_p%s'%(j+1)] #[USD] cost of one propeller
            
                # -------------------------------------------------------------
                # Checking the Minimum Arm Length for Given Propeller Diameter
                # -------------------------------------------------------------
                
                # [rad] The angle between two rotors
                delta_Theta = (2*math.pi)/n 
                
                # [mm] Minimum arm length before rotor contact
                minArmLen = ((d*1000)/2)/(math.sin(delta_Theta/2))
                                  
                # [mm] Required arm length (adding 15% of rotor diameter for clearance)                  
                ArmLen = minArmLen + 0.15*(d*1000) 
                
                # [mm] Clearance between adjacent rotors
                # when the arm length is set to the minimum arm length the clearance is zero
                clearance = 2*ArmLen*math.sin(delta_Theta/2)-(d*1000) 
            
                for k in range(numF):
                    print("\nFrame f%s"%(k+1))
                    print("------------")      
                
                    # Frame Parameters
                    nf = numRotors_frame_dict['n_f%s'%(k+1)] # number of rotors per frame
                    wb = wheelbase_frame_dict['wb_f%s'%(k+1)] #[mm] wheelbase
                    mf = mass_frame_dict['m_f%s'%(k+1)]/1000 #[kg] mass of frame
                    costf = cost_frame_dict['cost_f%s'%(k+1)] #[USD] cost of frame
                                   
                    # -----------------------------------------------
                    # Checking Compatibility of Frame and Propellers
                    # -----------------------------------------------
                    if ArmLen > wb:
                        print("Frame f%s is too small for propeller p%s"%(k+1,j+1))
                        pass
                    elif nf != n:
                        print("Frame f%s does not support %s rotors" %(k+1,n))
                    else:
                       
                        for l in range(numSC):
                            print("\nSpeed Controller sc%s"%(l+1))
                            print("------------") 
                        
                            # Speed Controller Parameters 
                            Vminsc = Vmin_speedcontroller_dict['Vmin_sc%s'%(l+1)]#[V] min voltage spd control
                            Vmaxsc = Vmax_speedcontroller_dict['Vmax_sc%s'%(l+1)]#[V] max voltage spd control
                            Imaxsc = Imax_speedcontroller_dict['Imax_sc%s'%(l+1)]#[V] max current spd control
                            msc = mass_speedcontroller_dict['m_sc%s'%(l+1)]/1000 #[kg] mass of one spd control
                            costsc = cost_speedcontroller_dict['cost_sc%s'%(l+1)] #[USD] cost one spd control
                            
                            # -------------------------------------------------------
                            # Checking Compatibility of Speed Controller and Battery
                            # -------------------------------------------------------
                            if not Vminsc <= Vb <= Vmaxsc:
                                print("Speed controller sc%s is not compatible with battery b%s"%(l+1,i+1))
                                pass
                            else:            
                            
                                for o in range(numM):
                                    print("\nMotor m%s"%(o+1))
                                    print("------------") 
                                    
                                    # Motor Parameters
                                    Kv = Kv_motor_dict['Kv_m%s'%(o+1)] #[rpm/V] constant velocity of the motor
                                    Vmaxm = Vmax_motor_dict['Vmax_m%s'%(o+1)] #[V] maximum voltage allowed for motor
                                    Imaxm = Imax_motor_dict['Imax_m%s'%(o+1)] #[A] maximum current allowed for motor
                                    mm = mass_motor_dict['m_m%s'%(o+1)]/1000 #[kg] mass of one motor
                                    costm = cost_motor_dict['cost_m%s'%(o+1)] #[USD] cost of one motor
                                    
                                
                                    # -----------------------------------------------------
                                    # Checking Compatibility of Motor and Speed Controller
                                    # -----------------------------------------------------
                                    if not Imaxsc >= Imaxm:
                                        print("Motor m%s is not compatible with speed controller sc%s"%(o+1,l+1))
                                        pass
                                    else:            
                                    
                                        # -----------------------------
                                        # Required Number of Batteries
                                        # -----------------------------
                                        eps1_func = lambda pitch: 2.317*math.exp(-11.212*pitch)
                                                                                 
                                        eps1 = eps1_func(pitch)
                                        if eps1 > 1:
                                            eps1 = 1
                                        else:
                                            pass
                                        
                                        eps2 = 2.537    
                                            
                                            
                                        # Flight time
                                        flgtTime = lambda nb: ((nb*cap)/(12.5*(mf+nb*mb+n*(mm+msc+mp)+mpld)*g*Kv*pitch*eps2*math.sqrt(eps1))) #[h] Flight time given actual number of batteriers
                                        
                                        # Number of batteries (isolate for nb from the above eqn.)
                                        if (cap-12.5*g*Kv*pitch*t*mb*eps2*math.sqrt(eps1)) > 0:
                                            nb = (12.5*(mf+n*(mm+msc+mp)+mpld)*g*Kv*pitch*t*eps2*math.sqrt(eps1))/(cap-12.5*g*Kv*pitch*t*mb*eps2*math.sqrt(eps1))
                                            
                                            # Motor Force Check
                                            M = mf + nb*mb + n*(mm+msc+mp) + mpld #[kg] total mass
                                            Tn = M*g/n #[N] thrust required per motor/propeller 
                                            N = (60/(pitch*d))*math.sqrt((8*Tn)/(eps1*rho*math.pi)) #[rpm] required propeller rotational speed to produced necessary thrust
                                            In = (eps2*Tn**(3/2)*Kv)/(N*d*math.sqrt((rho*math.pi)/2)) #[A] current to spin one propeller at speed N
                                            
                                            if In < Imaxm:
                                                actFlgtTime = flgtTime(nb) #[h] actual flight time given rounded number of batteries
                                                print("The required number of batteries is %d" %nb)
                                                print("The predicted flight time is %.2f [h]" %actFlgtTime)
                                                
                                                # -----------
                                                # Total Cost
                                                # -----------
                                                COST = nb*costb+costf+n*(costm+costp+costsc)
                                                print("The total cost of this UAV is $%.2f"%COST)
                                                solutions_dict['b%s, p%s, f%s, sc%s, m%s'%(i+1,j+1,k+1,l+1,o+1)] = (n, M, nb, actFlgtTime, COST)
                                                break
                                            else:
                                                print("Motor m%s doesn't produce enough force to lift the UAV"%(o+1))
                                        else:
                                             print("Motor m%s is not compatible with battery b%s"%(o+1,i+1))
                                                        
        if n == 4:
            solutions_dict_4 = solutions_dict
            n += 2
            # ---------------------------------------------------    
            # Identifying the Optimal Combination of Componenets
            # ---------------------------------------------------                              
            # The fourth index in the tuple is cost and the goal is to minimize the cost
            CHEAPESTold = 10000                         
            for key in solutions_dict_4:
                if solutions_dict_4[key][4] < CHEAPESTold: 
                    CHEAPEST = solutions_dict_4[key][4]
                    Best_4 = {key:solutions_dict_4[key]}
                    CHEAPESTold = CHEAPEST
                else:
                    pass                               
            
            BestCandidates_4 = {}                         
            for key in solutions_dict_4:      
                if abs(solutions_dict_4[key][4] - CHEAPEST) <= 5:
                    BestCandidates_4[key] = solutions_dict_4[key]
                else:
                    pass
        
        elif n == 6:
            solutions_dict_6 = solutions_dict
            n += 2
            # ---------------------------------------------------    
            # Identifying the Optimal Combination of Componenets
            # ---------------------------------------------------                              
            # The fourth index in the tuple is cost and the goal is to minimize the cost
            CHEAPESTold = 10000                         
            for key in solutions_dict_6:
                if solutions_dict_6[key][4] < CHEAPESTold: 
                    CHEAPEST = solutions_dict_6[key][4]
                    Best_6 = {key:solutions_dict_6[key]}
                    CHEAPESTold = CHEAPEST
                else:
                    pass                               
            
            BestCandidates_6 = {}                         
            for key in solutions_dict_6:      
                if abs(solutions_dict_6[key][4] - CHEAPEST) <= 5:
                    BestCandidates_6[key] = solutions_dict_6[key]
                else:
                    pass
            
        elif n == 8:
            solutions_dict_8 = solutions_dict
            n += 2
            # ---------------------------------------------------    
            # Identifying the Optimal Combination of Componenets
            # ---------------------------------------------------                              
            # The fourth index in the tuple is cost and the goal is to minimize the cost
            CHEAPESTold = 10000                         
            for key in solutions_dict_8:
                if solutions_dict_8[key][4] < CHEAPESTold: 
                    CHEAPEST = solutions_dict_8[key][4]
                    Best_8 = {key:solutions_dict_8[key]}
                    CHEAPESTold = CHEAPEST
                else:
                    pass                               
            
            BestCandidates_8 = {}                         
            for key in solutions_dict_8:      
                if abs(solutions_dict_8[key][4] - CHEAPEST) <= 150:
                    BestCandidates_8[key] = solutions_dict_8[key]
                else:
                    pass
            
    # Note:
    # solutions_dict is ogranized such that the key is in the form:
    # b#, f#, p#, sc#, m# for the battery, frame, propeller, speed controller, and 
    # motor part numbers respectively
    # The values shown for any given key of solutions_dict are in a tuple and are:
    # (n, M, nbRnd, actFlgtTime, COST) aka (number of rotors, total mass [kg], number of
    # batteries, the actual flight time [h], and the total cost [USD])
         
    
    # --------------------------------------
    # Outputting the Results to a Text File
    # --------------------------------------
    outputFile = open(r"Best_Candidates(rev3).txt", "w")
    str1 = 'Best Candidates\n###############\n'
    
    count = 1
    str2 = ''
    for key in BestCandidates_4:#**************************************************
        strloop = """\nOption %d:\n
        Components: %s
        Number of rotors: %d
        Total mass (including payload): %.2f [kg]
        Mass (excluding payload): %.2f [kg]
        Number of batteries: %.1f
        Predicted flight time: %.2f [h]
        Total cost: $%.2f [USD]
        """%(count, key, BestCandidates_4[key][0],
        BestCandidates_4[key][1], BestCandidates_4[key][1]-mpld, 
        BestCandidates_4[key][2],BestCandidates_4[key][3],BestCandidates_4[key][4])
        str2 += strloop
        
        # Plotting a bar chart for each of the best options
        x_componenets = ('Battery\n(x%s)'%math.floor(BestCandidates_4[key][2]),'Propeller\n(x4)',
                         'Frame\n(x1)', 'Speed\nController\n(x4)', 'Motor\n(x4)')
        y_cost = [i for i in range(len(x_componenets))]
        performance = [cost_battery_dict['cost_%s'%(key.split(', ')[0])]*BestCandidates_4[key][2], 
                        cost_propeller_dict['cost_%s'%(key.split(', ')[1])]*4,
                        cost_frame_dict['cost_%s'%(key.split(', ')[2])],
                        cost_speedcontroller_dict['cost_%s'%(key.split(', ')[3])]*4,
                        cost_motor_dict['cost_%s'%(key.split(', ')[4])]*4]
        plt.bar(y_cost, performance, align='center', alpha=0.5)
        plt.xticks(y_cost, x_componenets)
        plt.ylabel('Cost [USD]')
        plt.title('Option %d Cost Breakdown' %count)
        plt.show()
        
        count +=1
    
    str3 = ''
    for key in BestCandidates_6:
        strloop = """\nOption %d:\n
        Components: %s
        Number of rotors: %d
        Total mass (including payload): %.2f [kg]
        Mass (excluding payload): %.2f [kg]
        Number of batteries: %d
        Predicted flight time: %.2f [h]
        Total cost: $%.2f [USD]
        """%(count, key, BestCandidates_6[key][0],
        BestCandidates_6[key][1], BestCandidates_6[key][1]-mpld, 
        BestCandidates_6[key][2],BestCandidates_6[key][3],BestCandidates_6[key][4])
        str3 += strloop
        
        # Plotting a bar chart for each of the best options
        x_componenets = ('Battery\n(x%s)'%math.floor(BestCandidates_6[key][2]),'Propeller\n(x6)',
                         'Frame\n(x1)', 'Speed\nController\n(x6)', 'Motor\n(x6)')
        y_cost = [i for i in range(len(x_componenets))]
        performance = [cost_battery_dict['cost_%s'%(key.split(', ')[0])]*BestCandidates_6[key][2], 
                        cost_propeller_dict['cost_%s'%(key.split(', ')[1])]*4,
                        cost_frame_dict['cost_%s'%(key.split(', ')[2])],
                        cost_speedcontroller_dict['cost_%s'%(key.split(', ')[3])]*4,
                        cost_motor_dict['cost_%s'%(key.split(', ')[4])]*4]
        plt.bar(y_cost, performance, align='center', alpha=0.5)
        plt.xticks(y_cost, x_componenets)
        plt.ylabel('Cost [USD]')
        plt.title('Option %d Cost Breakdown' %count)
        plt.show()
        
        count +=1
        
    str4 = ''
    for key in BestCandidates_8:
        strloop = """\nOption %d:\n
        Components: %s
        Number of rotors: %d
        Total mass (including payload): %.2f [kg]
        Mass (excluding payload): %.2f [kg]
        Number of batteries: %d
        Predicted flight time: %.2f [h]
        Total cost: $%.2f [USD]
        """%(count, key, BestCandidates_8[key][0],
        BestCandidates_8[key][1], BestCandidates_8[key][1]-mpld, 
        BestCandidates_8[key][2],BestCandidates_8[key][3],BestCandidates_8[key][4])
        str4 += strloop
        
        # Plotting a bar chart for each of the best options
        x_componenets = ('Battery\n(x%s)'%math.floor(BestCandidates_8[key][2]),'Propeller\n(x8)',
                         'Frame\n(x1)', 'Speed\nController\n(x8)', 'Motor\n(x8)')
        y_cost = [i for i in range(len(x_componenets))]
        performance = [cost_battery_dict['cost_%s'%(key.split(', ')[0])]*BestCandidates_8[key][2], 
                       cost_propeller_dict['cost_%s'%(key.split(', ')[1])]*4,
                       cost_frame_dict['cost_%s'%(key.split(', ')[2])],
                       cost_speedcontroller_dict['cost_%s'%(key.split(', ')[3])]*4,
                       cost_motor_dict['cost_%s'%(key.split(', ')[4])]*4]
        plt.bar(y_cost, performance, align='center', alpha=0.5)
        plt.xticks(y_cost, x_componenets)
        plt.ylabel('Cost [USD]')
        
        if count == 21 and mpld == 2.5 and g == 9.81 and rho == 1.225 and t == 24.6/60:
            plt.title('Dronolab Cost Breakdown')
             
        else:
            plt.title('Option %d Cost Breakdown' %count)
        plt.show()
        
        count +=1
    
    string = [str1,str2, str3, str4 ]
    outputFile.writelines(string)
    
    outputFile.close()
    
              
    solutions_dict_all = {**solutions_dict_4,**solutions_dict_6,**solutions_dict_8}
    #solutions_dict_all.update(solutions_dict_4,solutions_dict_6,solutions_dict_8)
    
    numIter = [i for i in range(len(solutions_dict_4)+len(solutions_dict_6)+len(solutions_dict_8))]
    
    
    #figure1
    plt.plot(numIter, [i[4] for i in list(solutions_dict_all.values())], '.k')
    plt.xlabel('Number of Iterations\n(Viable Solutions of %d Possible Combinations)'%(numB*numF*numP*numSC*numM))
    plt.ylabel('Cost [USD]')
    plt.show()
    
    
    
    # #figure2
    # x_componenets = ('Battery', 'Propeller', 'Frame', 'Speed Controller', 'MOtor')
    # y_cost = [i for i in range(len(x_componenets))]
    # performance = [cost_battery_dict['cost_%s'%(key[0:2])]*BestCandidates_8[key][2], 
    #                cost_propeller_dict['cost_%s'%(key[4:6])]*4,
    #                cost_frame_dict['cost_%s'%(key[8:10])],
    #                cost_speedcontroller_dict['cost_%s'%(key[12:15])]*4,
    #                cost_motor_dict['cost_%s'%(key[17:19])]*4]
    # plt.bar(y_cost, performance, align='center', alpha=0.5)
    # plt.xticks(y_cost, x_componenets)
    # plt.ylabel('Cost [USD]')
    # plt.title('Option %d Cost Breakdown' %count)
    # plt.show()
    
    
    
    stop = timeit.default_timer() # Stopping the run time timer
    print('\n\nRuntime: %.4f [s]' %(stop - start))  
    
    endLog = datetime.now()
    dtStart_string = startLog.strftime("%b-%d-%Y %H:%M:%S")
    dtEnd_string = endLog.strftime("%b-%d-%Y %H:%M:%S")
    #print("date and time =", dtStart_string)
    
    logFile = open(r"log(rev3).txt", "a")
    log = """\n
*******************************************
Run start: %s
Run stop: %s
Runtime: %.4f [s] 
%d combinations completed succesfully
"""%(dtStart_string,dtEnd_string,(stop - start), numB*numF*numM*numP*numSC)
    
    logFile.writelines(log)
    
    logFile.close()

except: 
    stop = timeit.default_timer() # Stopping the runtime timer
    endLog = datetime.now()
    
    print('\n\nRuntime: %.4f [s]' %(stop - start))  
    print('Failed to complete run\n',traceback.format_exc())
    
    dtStart_string = startLog.strftime("%b-%d-%Y %H:%M:%S")
    dtEnd_string = endLog.strftime("%b-%d-%Y %H:%M:%S")
    #print("date and time =", dtStart_string)
    
    logFile = open(r"log.txt", "a")
    log = """\n
*******************************************
Run start: %s
Run stop: %s
Runtime: %.4f [s] 
Failed to complete run
"""%(dtStart_string,dtEnd_string,(stop - start))
      
    logFile.writelines([log,traceback.format_exc()])
    
    logFile.close()



