# -*- coding: utf-8 -*-
"""
Created on Tue Jan 19 14:50:35 2021

@author: Mateus A. R. Braga

Reading the parameters from the off-the-shelf components library and defining a
dictionary for each type of component and parameter
"""

#-----------
# Batteries
#-----------
# There are four parameters of impotance for the batteries. They include the 
# battery capacity, the battery voltage, the battery mass, and the cost of the 
# battery.

# The keys needed to call the parameters from the dictionary all begin with 
# cap, mb, or costb respectively. To call a specific value, say for the 
# capacity for battery b3 (its item number is b3), then you add an underscore
# and then the item number to the key: capacity_battery_dict['cap_b3']

# Dictionaries: capacity_battery_dict; voltage_battery_dict; mass_battery_dict; 
#               cost_battery_dict    
# Keys: cap_b# ; V_b# ; m_b# ; cost_b#

OTSC_library_batteries = open(r"F:\JUVS paper\OTSC Library\OTSC_library_batteries.txt","r") 

rows = OTSC_library_batteries.readlines()

itemNo = [0 for x in range(len(rows)-1)]
cap = [0 for x in range(len(rows)-1)]
VoltB = [0 for x in range(len(rows)-1)]
mb = [0 for x in range(len(rows)-1)]
costb = [0 for x in range(len(rows)-1)]
for i in range(len(rows)-1):
    split_rows = rows[i+1].split('\t')
    itemNo[i] = split_rows[0]
    cap[i] = float(split_rows[1]) #[mAh] Battery capacity
    VoltB[i] = float(split_rows[2]) #[V] Battery voltage
    mb[i] = float(split_rows[3]) #[g] Battery mass
    costb[i] = float(split_rows[4]) #[$] Cost in USD

OTSC_library_batteries.close()

capacity_battery_dict = {} #[mAh] Battery capacity (dictionary)
keys_cap = ['cap_'+s for s in itemNo]
for i in range(len(keys_cap)):
    capacity_battery_dict[keys_cap[i]] = cap[i]
  
voltage_battery_dict = {} #[V] Battery voltage (dictionary)
keys_V = ['V_'+s for s in itemNo]
for i in range(len(keys_V)):
    voltage_battery_dict[keys_V[i]] = VoltB[i]    
    
mass_battery_dict = {} #[g] Battery mass (dictionary)
keys_mb = ['m_'+s for s in itemNo]
for i in range(len(keys_mb)):
    mass_battery_dict[keys_mb[i]] = mb[i]
 
cost_battery_dict = {} #[$] Cost in USD (dictionary)
keys_costb = ['cost_'+s for s in itemNo]
for i in range(len(keys_costb)):
    cost_battery_dict[keys_costb[i]] = costb[i]    


#--------
# Frames
#--------
# There are four parameters of impotance for the frames. They include the 
# number of rotors, the wheelbase, the mass, and the cost.

# Dictionaries: numRotors_frame_dict; wheelbase_frame_dict; mass_frame_dict; 
#               cost_frame_dict   
# Keys: n_f# ; wb_f# ; m_f# ; cost_f#

OTSC_library_frames = open(r"F:\JUVS paper\OTSC Library\OTSC_library_frames.txt","r") 

rows = OTSC_library_frames.readlines()

itemNo = [0 for x in range(len(rows)-1)]
nf = [0 for x in range(len(rows)-1)]
wb = [0 for x in range(len(rows)-1)]
mf = [0 for x in range(len(rows)-1)]
costf = [0 for x in range(len(rows)-1)]
for i in range(len(rows)-1):
    split_rows = rows[i+1].split('\t')
    itemNo[i] = split_rows[0]
    nf[i] = float(split_rows[1]) # Number of rotors per frame
    wb[i] = float(split_rows[2]) #[mm] Wheelbase
    mf[i] = float(split_rows[3]) #[g] Frame mass
    costf[i] = float(split_rows[4]) #[$] Cost in USD

OTSC_library_frames.close()

numRotors_frame_dict = {} # Number of rotors per frame (dictionary)
keys_nf = ['n_'+s for s in itemNo]
for i in range(len(keys_nf)):
    numRotors_frame_dict[keys_nf[i]] = nf[i]
    
wheelbase_frame_dict = {} #[mm] Wheelbase (dictionary)
keys_wb = ['wb_'+s for s in itemNo]
for i in range(len(keys_wb)):
    wheelbase_frame_dict[keys_wb[i]] = wb[i]
 
    
mass_frame_dict = {} #[g] Frame mass (dictionary)
keys_mf = ['m_'+s for s in itemNo]
for i in range(len(keys_mf)):
    mass_frame_dict[keys_mf[i]] = mf[i]
 
cost_frame_dict = {} #[$] Cost in USD (dictionary)
keys_costf = ['cost_'+s for s in itemNo]
for i in range(len(keys_costf)):
    cost_frame_dict[keys_costf[i]] = costf[i]    
 
    
#--------
# Motors
#--------
# There are five parameters of impotance for the motors. They include the 
# Kv, the max current, the max voltage, the mass, and the cost
    
# Dictionaries: Kv_motor_dict; Imax_motor_dict; Vmax_motor_dict; 
#               mass_motor_dict; cost_motor_dict   
# Keys: Kv_m# ; Imax_m# ; Vmax_m# ; m_m#; cost_m#

OTSC_library_motors = open(r"F:\JUVS paper\OTSC Library\OTSC_library_motors.txt","r") 

rows = OTSC_library_motors.readlines()

itemNo = [0 for x in range(len(rows)-1)]
Kv = [0 for x in range(len(rows)-1)]
Imaxm = [0 for x in range(len(rows)-1)]
Vmaxm = [0 for x in range(len(rows)-1)]
mm = [0 for x in range(len(rows)-1)]
costm = [0 for x in range(len(rows)-1)]
for i in range(len(rows)-1):
    split_rows = rows[i+1].split('\t')
    itemNo[i] = split_rows[0]
    Kv[i] = float(split_rows[1]) #[rpm/V] 
    Imaxm[i] = float(split_rows[2]) #[A] Maximum current for the motor
    Vmaxm[i] = float(split_rows[3]) #[V] Maximum voltage for the motor
    mm[i] = float(split_rows[4]) #[g] Motor mass
    costm[i] = float(split_rows[5]) #[$] Cost in USD

OTSC_library_motors.close()

Kv_motor_dict = {} #[rpm/V] (dictionary)
keys_Kv = ['Kv_'+s for s in itemNo]
for i in range(len(keys_Kv)):
    Kv_motor_dict[keys_Kv[i]] = Kv[i]
    
Imax_motor_dict = {} #[A] Maximum current for the motor (dictionary)
keys_Imaxm = ['Imax_'+s for s in itemNo]
for i in range(len(keys_Imaxm)):
    Imax_motor_dict[keys_Imaxm[i]] = Imaxm[i]
    
Vmax_motor_dict = {} #[V] Maximum voltage for the motor (dictionary)
keys_Vmaxm = ['Vmax_'+s for s in itemNo]
for i in range(len(keys_Vmaxm)):
    Vmax_motor_dict[keys_Vmaxm[i]] = Vmaxm[i]
    
mass_motor_dict = {} #[g] Battery mass (dictionary)
keys_mm = ['m_'+s for s in itemNo]
for i in range(len(keys_mm)):
    mass_motor_dict[keys_mm[i]] = mm[i]
 
cost_motor_dict = {} #[$] Cost in USD (dictionary)
keys_costm = ['cost_'+s for s in itemNo]
for i in range(len(keys_costm)):
    cost_motor_dict[keys_costm[i]] = costm[i]   
    
  
#------------
# Propellers
#------------
# There are four parameters of impotance for the propellers. They include the 
# diameter, pitch, mass, and cost
    
# Dictionaries: diameter_propeller_dict; pitch_propeller_dict;  
#               mass_propeller_dict; cost_propeller_dict   
# Keys: d_p# ; pitch_p# ; m_p# ; cost_p#

OTSC_library_propellers = open(r"F:\JUVS paper\OTSC Library\OTSC_library_propellers.txt","r") 

rows = OTSC_library_propellers.readlines()

itemNo = [0 for x in range(len(rows)-1)]
d = [0 for x in range(len(rows)-1)]
pitch = [0 for x in range(len(rows)-1)]
mp = [0 for x in range(len(rows)-1)]
costp = [0 for x in range(len(rows)-1)]
for i in range(len(rows)-1):
    split_rows = rows[i+1].split('\t')
    itemNo[i] = split_rows[0]
    d[i] = float(split_rows[1]) #[in] Propeller diameter
    pitch[i] = float(split_rows[2]) #[in] Propeller pitch
    mp[i] = float(split_rows[3]) #[g] Propeller mass
    costp[i] = float(split_rows[4]) #[$] Cost in USD

OTSC_library_propellers.close()

diameter_propeller_dict = {} #[in] Propeller diameter (dictionary)
keys_d = ['d_'+s for s in itemNo]
for i in range(len(keys_d)):
    diameter_propeller_dict[keys_d[i]] = d[i]
    
pitch_propeller_dict = {} #[in] Propeller pitch (dictionary)
keys_pitch = ['pitch_'+s for s in itemNo]
for i in range(len(keys_pitch)):
    pitch_propeller_dict[keys_pitch[i]] = pitch[i]
 
    
mass_propeller_dict = {} #[g] Propeller mass (dictionary)
keys_mp = ['m_'+s for s in itemNo]
for i in range(len(keys_mp)):
    mass_propeller_dict[keys_mp[i]] = mp[i]
 
cost_propeller_dict = {} #[$] Cost in USD (dictionary)
keys_costp = ['cost_'+s for s in itemNo]
for i in range(len(keys_costp)):
    cost_propeller_dict[keys_costp[i]] = costp[i]        
    
  
#-------------------
# Speed Controllers
#-------------------
# There are five parameters of impotance for the speed controllers. They 
# include the min voltage, the max voltage, the max current, the mass, and cost
    
# Dictionaries: Vmin_speedcontroller_dict; Vmax_speedcontroller_dict;  
#               Imax_speedcontroller_dict; mass_speedcontroller_dict;
#               cost_speedcontroller_dict
# Keys: Vmin_sc# ; Vmax_sc# ; Imax_sc# ; m_sc#; cost_sc#

OTSC_library_speedcontrollers = open(r"F:\JUVS paper\OTSC Library\OTSC_library_speedcontrollers.txt","r") 

rows = OTSC_library_speedcontrollers.readlines()

itemNo = [0 for x in range(len(rows)-1)]
Vminsc = [0 for x in range(len(rows)-1)]
Vmaxsc = [0 for x in range(len(rows)-1)]
Imaxsc = [0 for x in range(len(rows)-1)]
msc = [0 for x in range(len(rows)-1)]
costsc = [0 for x in range(len(rows)-1)]
for i in range(len(rows)-1):
    split_rows = rows[i+1].split('\t')
    itemNo[i] = split_rows[0]
    Vminsc[i] = float(split_rows[1]) #[V] Min voltage for the speed controller
    Vmaxsc[i] = float(split_rows[2]) #[V] Max voltage for the speed controller
    Imaxsc[i] = float(split_rows[3]) #[A] Max current for the speed controller
    msc[i] = float(split_rows[4]) #[g] Speed controller mass
    costsc[i] = float(split_rows[5]) #[$] Cost in USD

OTSC_library_speedcontrollers.close()

Vmin_speedcontroller_dict = {} #[V] Minimum voltage for the speed controller 
                                                                 # (dictionary)
keys_Vminsc = ['Vmin_'+s for s in itemNo]
for i in range(len(keys_Vminsc)):
    Vmin_speedcontroller_dict[keys_Vminsc[i]] = Vminsc[i]
     
Vmax_speedcontroller_dict = {} #[V] Maximum voltage for the speed controller 
                                                                 # (dictionary)
keys_Vmaxsc = ['Vmax_'+s for s in itemNo]
for i in range(len(keys_Vmaxsc)):
    Vmax_speedcontroller_dict[keys_Vmaxsc[i]] = Vmaxsc[i]
    
Imax_speedcontroller_dict = {} #[A] Maximum current for the speed controller 
                                                                 # (dictionary)
keys_Imaxsc = ['Imax_'+s for s in itemNo]
for i in range(len(keys_Imaxsc)):
    Imax_speedcontroller_dict[keys_Imaxsc[i]] = Imaxsc[i]
    
mass_speedcontroller_dict = {} #[g] Speed controller mass (dictionary)
keys_msc = ['m_'+s for s in itemNo]
for i in range(len(keys_msc)):
    mass_speedcontroller_dict[keys_msc[i]] = msc[i]
 
cost_speedcontroller_dict = {} #[$] Cost in USD (dictionary)
keys_costsc = ['cost_'+s for s in itemNo]
for i in range(len(keys_costsc)):
    cost_speedcontroller_dict[keys_costsc[i]] = costsc[i]  
    
    
    

# for i in range(len(keys_costb)):
#     print(cost_battery_dict['cost_b%s'%(i+1)]) 

# print('\n\n')
# for i in range(len(keys_costf)):
#     print(cost_frame_dict['cost_f%s'%(i+1)]) 
    
# print('\n\n')
# for i in range(len(keys_costm)):
#     print(cost_motor_dict['cost_m%s'%(i+1)]) 
    
# print('\n\n')
# for i in range(len(keys_costp)):
#     print(cost_propeller_dict['cost_p%s'%(i+1)]) 
    
# print('\n\n')
# for i in range(len(keys_costsc)):
#     print(cost_speedcontroller_dict['cost_sc%s'%(i+1)]) 